# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#
import logging

from climetlab.core.data import get_data_entry

LOG = logging.getLogger(__name__)

areas = {
    "austria": (55.5, 6.0, 40.0, 21.5),
    "azores": (46.0, -35.0, 30.5, -19.5),
    "balearic islands": (47.5, -4.5, 32.0, 11.0),
    "belgium": (58.5, -3.0, 43.0, 12.5),
    "bulgaria": (50.5, 18.0, 35.0, 33.5),
    "canary islands": (36.5, -23.0, 21.0, -7.5),
    "corsica": (50.0, 1.5, 34.5, 17.0),
    "croatia": (52.5, 9.0, 37.0, 24.5),
    "czech republic": (58.0, 8.0, 42.5, 23.5),
    "denmark": (64.0, 3.0, 48.5, 18.5),
    "estonia": (66.5, 17.5, 51.0, 33.0),
    "finland": (73.0, 18.5, 57.5, 34.0),
    "france": (54.5, -6.0, 39.0, 9.5),
    "germany": (59.0, 3.0, 43.5, 18.5),
    "greece": (46.5, 16.5, 31.0, 32.0),
    "hungary": (55.0, 12.0, 39.5, 27.5),
    "iceland": (73.0, -26.0, 57.5, -10.5),
    "ireland": (61.5, -15.0, 46.0, 0.5),
    "israel": (39.5, 27.5, 24.0, 43.0),
    "italy": (50.5, 5.0, 35.0, 20.5),
    "latvia": (65.0, 17.0, 49.5, 32.5),
    "lithuania": (63.0, 16.5, 47.5, 32.0),
    "luxembourg": (58.0, -1.5, 42.5, 14.0),
    "madeira": (41.0, -24.0, 25.5, -8.5),
    "montenegro": (50.5, 12.0, 35.0, 27.5),
    "morocco": (36.5, -16.0, 21.0, -0.5),
    "netherlands": (60.0, -2.5, 44.5, 13.0),
    "north macedonia": (49.5, 14.0, 34.0, 29.5),
    "norway": (73.0, 4.5, 57.5, 20.0),
    "portugal": (47.5, -15.0, 32.0, 0.5),
    "serbia": (52.0, 13.5, 36.5, 29.0),
    "romania": (54.0, 17.5, 38.5, 33.0),
    "sardinia": (48.0, 1.5, 32.5, 17.0),
    "sicily": (45.5, 6.5, 30.0, 22.0),
    "slovakia": (56.5, 12.0, 41.0, 27.5),
    "slovenia": (54.0, 7.5, 38.5, 23.0),
    "spain": (48.0, -10.0, 32.5, 5.5),
    "sweden": (70.0, 10.0, 54.5, 25.5),
    "switzerland": (55.0, 0.5, 39.5, 16.0),
    "turkey": (47.0, 25.5, 31.5, 41.0),
    "united kingdom": (63.5, -10.0, 48.0, 5.5),
}
areas.update(
    {
        "uk": areas["united kingdom"],
        "republic of serbia": areas["serbia"],
    }
)


def _update_areas(old, new, prefix=""):
    for name, values in new.items():
        name = prefix + name
        assert name not in old, f"{name} already defined."
        if len(values) > 1:
            LOG.debug(
                f"Area {name} has multiple values {values}. Not supported by CliMetLab."
            )
            continue
        old[name] = values[0]


data = get_data_entry("domains", "verification").data
_update_areas(areas, data["areas"], prefix="verification.")

AREAS = {k: tuple(v) for k, v in areas.items()}
AREAS_LONG_NAMES = {"verification." + k: v for k, v in data["areas_long_names"].items()}


def domain_to_area(name):
    if isinstance(name, (list, tuple)):
        return name
    return AREAS[name.lower()]


def domain_to_area_long_name(name):
    return AREAS_LONG_NAMES.get(name.lower())



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

from climetlab.core.data import data_entries, get_data_entry
from climetlab.core.ipython import display
from climetlab.core.settings import SETTINGS
from climetlab.wrappers import get_wrapper

from .options import Options

OPTIONS = {}


def magics(*args, **kwargs):
    from .backends.magics.backend import Backend as MagicsBackend

    return MagicsBackend(*args, **kwargs)


def bokeh(*args, **kwargs):
    from .backends.bokeh.backend import Backend as BokehBackend

    return BokehBackend(*args, **kwargs)


def matplotlib(*args, **kwargs):
    from .backends.matplotlib.backend import Backend as MatplotlibBackend

    return MatplotlibBackend(*args, **kwargs)


DRIVERS = {
    None: magics,
    "magics": magics,
    "matplotlib": matplotlib,
    "bokeh": bokeh,
}


def options(**kwargs):
    global OPTIONS
    OPTIONS = kwargs


def projection(name):
    return get_data_entry("projections", name)


def layer(name):
    return get_data_entry("layers", name)


def style(name):
    return get_data_entry("styles", name)


def projections():
    return sorted(e.name for e in data_entries("projections"))


def layers():
    return sorted(e.name for e in data_entries("layers"))


def styles():
    return sorted(e.name for e in data_entries("styles"))


class Plot:
    """[summary]"""

    def __init__(self, kwargs):
        backend = SETTINGS.get(f"{self.kind}-plotting-backend", None)
        backend = kwargs.pop("backend", backend)

        options = {}
        options.update(SETTINGS.get("plotting-options", {}))
        options.update(OPTIONS)
        options.update(kwargs)
        self.backend = DRIVERS[backend](Options(options))

    def plot_graph(self, data=None, **kwargs):

        if not isinstance(data, (list, tuple)):
            data = [data]

        for d in data:
            d = get_wrapper(d)
            d.plot_graph(self.backend)

        options = Options(kwargs)
        self.backend.apply_options(options)
        options.check_unused()

        return self

    def plot_map(self, data=None, **kwargs):

        if not isinstance(data, (list, tuple)):
            data = [data]

        for d in data:
            d = get_wrapper(d)
            d.plot_map(self.backend)

        options = Options(kwargs)
        self.backend.apply_options(options)
        options.check_unused()

        return self

    def wms_layers(self):
        return self.backend.wms_layers()

    def show(self):
        self.backend.show(display=display)

    def macro(self) -> list:
        return self.backend.macro()

    def save(self, path):
        return self.backend.save(path)


class MapPlot(Plot):
    kind = "map"


class GraphPlot(Plot):
    kind = "graph"


def new_plot(**kwargs) -> Plot:
    """[summary]

    :return: [description]
    :rtype: Plot
    """
    return MapPlot(kwargs)


def new_graph(**kwargs) -> Plot:
    """[summary]

    :return: [description]
    :rtype: Plot
    """
    return GraphPlot(kwargs)


def plot_graph(data=None, **kwargs):
    """Plot other-than-map data

    Args:
        data ([any]): [description]
    """

    p = new_graph(**kwargs)
    p.plot_graph(data)
    p.show()


def plot_map(data=None, **kwargs):
    """Plot any data on a map.

    Args:
        data ([any]): [description]
    """

    p = new_plot(**kwargs)
    p.plot_map(data)
    p.show()


Plot.plot_map.__doc__ = plot_map.__doc__


def interactive_map(data=None, **kwargs):
    from climetlab.plotting.wms import interactive_map as wms_map

    return wms_map(data, **kwargs)



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

from .arguments.transformers import ALL
from .core.caching import CACHE as cache
from .core.initialise import initialise
from .core.settings import SETTINGS as settings
from .datasets import Dataset
from .datasets import get_dataset as dataset  # so the user can do: cml.dataset(...)
from .datasets import load_dataset
from .plotting import interactive_map, new_plot
from .plotting import options as plotting_options
from .plotting import plot_graph, plot_map
from .readers import Reader
from .sources import Source
from .sources import get_source as source  # so the user can do: cml.source(...)
from .sources import load_source, load_source_lazily
from .version import __version__
from .wrappers import Wrapper

__all__ = [
    "ALL",
    "cache",
    "dataset",
    "Dataset",
    "Wrapper",
    "interactive_map",
    "load_dataset",
    "load_source",
    "load_source_lazily",
    "new_plot",
    "plot_graph",
    "plot_map",
    "plotting_options",
    "Reader",
    "settings",
    "source",
    "Source",
    "__version__",
]


initialise()



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#


class Backend:
    def __init__(self, options):

        self._options = options

        from matplotlib.pyplot import figure

        self.figure = figure()
        self.ax = self.figure.add_subplot(1, 1, 1)

    # def finalize2(self):

    #     state = State(self)
    #     for a in self._actions:
    #         state = a.tranform(state)
    def plot_graph_add_timeserie(self, frame):
        frame.plot(x="date", ax=self.ax)

    def plot_graph_pandas(self, frame, time: str, variable: str):
        k = self._options("k", "pandas")

        import pandas

        frame.date = pandas.to_datetime(frame.date)

        # TODO: need to set variable ?
        # print(time, variable)

        if k == "seaborn":
            import seaborn as sns

            sns.lineplot(data=frame, x="date", y="value")

            return

        if k == "pandas":
            kwargs = self._options("pandas_dataframe_plot_kwargs", {})

            frame.plot(x="date", ax=self.ax, **kwargs)
            return

    def option(self, name, default=None):
        return self._options(name, default)

    def apply_options(self, options):
        pass

    def show(self, display):
        self.figure.show()  # not usefull in jupyter lab
        # display(self.figure)

    def save(self, path):
        self.figure.savefig(path)
